#!/usr/bin/env python3

import os, json, sys, time, logging, traceback
from datetime import datetime
from contextlib import contextmanager

args = sys.argv[1:] # List of arguments that were passed

yes_for_all = True if '--yes-for-all' in args else False
log_dir = args[args.index('--log-dir')+1] if '--log-dir' in args else 'logs' # The directory where the log files will be saved

# Declaring and configuring the logger
class FileHandlerNotEmpty(logging.FileHandler):
    def __init__(self, filename, mode='a', encoding=None, delay=False):
        self.fname = filename
        self.has_error = False
        super().__init__(filename, mode, encoding, delay)

    def emit(self, record):
        if record.levelno >= logging.ERROR:
            self.has_error = True
        super().emit(record)

    def close(self):
        super().close()
        if not self.has_error:
            os.remove(self.fname)

now = datetime.now().strftime("%Y-%m-%d_-_%H_%M")
log_file_name = f'{now}.log'
logger = logging.getLogger()
logger.setLevel(logging.INFO)
formatter = logging.Formatter('%(asctime)s - %(levelname)s: %(message)s')
file_handler = FileHandlerNotEmpty(f'{log_dir}/{log_file_name}')
console_handler = logging.StreamHandler(sys.stdout)
file_handler.setFormatter(formatter)
file_handler.setLevel(logging.ERROR)
console_handler.setFormatter(formatter)
logger.addHandler(file_handler)
logger.addHandler(console_handler)

debug = lambda msg: logger.debug(msg)
info = lambda msg: logger.info(msg)
warning = lambda msg: logger.warning(msg)
error = lambda msg: logger.error(msg)
critical_error = lambda msg: logger.critical(msg)

def confirm(text:str)->bool:
    """
    Checks if a file exists in the current directory.

    Arguments:
        file (str): The name of the file to check.

    Returns:
        bool: True if the file exists, False otherwise.
    """

    confirm = 'y' if yes_for_all else input(text)
    
    if confirm.lower() not in ('n', 'no'):
        return True
    else:
        return False

def check_file(path_to_file:str)->bool:
    """
    Checks if a file exists in the current directory.

    Arguments:
        file (str): The name of the file to check.

    Returns:
        bool: True if the file exists, False otherwise.
    """

    if os.path.isfile('./' + path_to_file):
        return True
    else:
        return False

def ask_for_data(required_data:tuple, file_name_no_extension:str, ask:bool=True)->dict:
    """
    Asks the user for input, creates a JSON configuration file (.conf), and returns the input as a dictionary.

    Args:
        required_data (tuple): The data to be collected from the user.
        file_name_no_extension (str): The name of the configuration file without the extension.
        ask (bool, optional): Whether to ask for user input or use default values. Defaults to True.

    Returns:
        dict: A dictionary containing the collected data.
    """

    data_dict = {}
    if ask:
        for data in required_data:
            data_dict[data] = input(f'Enter the {data}: ')

        json.dump(data_dict, open(f'config/{file_name_no_extension}.conf', 'w'))
    else:
        for data in required_data:
            data_dict[data[0]] = data[1]

        json.dump(data_dict, open(f'config/{file_name_no_extension}.conf', 'w'))

    return data_dict

def list_from_input(text:str)->list:
    """
    Converts a comma-separated user input string into a list.

    Arguments:
        text (str): The text to be displayed as a prompt for user input.

    Returns:
        list: A list containing the user input as integers or strings.
    """

    raw_list = input(text)

    try:
        final_list = [int(n) for n in raw_list.replace(' ', '').split(',')]
    except:
        final_list = [str(s) for s in raw_list.replace(' ', '').split(',')]

    return final_list

@contextmanager # A function that creates generators (and use them to iterate instead of lists of web elements) was one of the ways that I found to prevent memory leak when unsing Selenium
def generator(list_:list):
    """Creates a generator from a list"""
   
    yield list_



#!/usr/bin/env python3

# Importing from modules / packages
from selenium import webdriver
from selenium.webdriver.firefox.options import Options
from selenium.webdriver.common.by import By
from selenium.webdriver.support.wait import WebDriverWait
from selenium.webdriver.support.select import Select
from selenium.webdriver.common.action_chains import ActionChains
from selenium.webdriver.support import expected_conditions as EC
from .basic_functions import *

headless = False if '--show-browser' in args else True if '--headless' in args else confirm('Do you want the browser to be headless (i.e., you won\'t see it)? [y/n] ')

def get_webdriver(headless:bool)->object:
    """
    Sets up and returns a selenium webdriver object with desired configurations. Uses GeckoDriver (Firefox).

    Arguments:
        headless (bool): Whether to run the webdriver in headless mode.

    Returns:
        object: An instance of webdriver.Firefox with specified options.
    """

    # Declaring some important variables for the Gecko Webdriver (Firefox)
    options = Options()
    options.set_preference("browser.download.folderList", 2)
    options.set_preference("browser.download.manager.showWhenStarting", False)
    # options.set_preference("browser.download.dir", DOWNLOAD_DIRECTORY)
    options.set_preference("browser.helperApps.neverAsk.saveToDisk", "application/x-gzip")
    options.add_argument('disable-infobars')
    options.add_argument('--disable-extensions')
    #options.add_argument(f'user-agent={linux_useragent}')
    options.add_argument('--no-sandbox')

    if headless is True:
        options.add_argument('--headless')

    driver = webdriver.Firefox(options = options)
    driver.set_window_size(1920,1040)

    return driver

def scroll_to(driver:object, element:object)->None:
    """
    Scrolls the view of a given webdriver to a specific web element.

    Arguments:
        driver (object): The webdriver instance to scroll.
        element (object): The web element to scroll to.
        
    Returns:
        None
    """

    time.sleep(.05)
    x = element.location['x']
    y = element.location['y']
    scroll_by_coord = 'window.scrollTo(%s,%s);' % (x,y)
    scroll_nav_out_of_way = 'window.scrollBy(0, -120);'
    try:
        driver.execute_script(scroll_by_coord)
        driver.execute_script(scroll_nav_out_of_way)
    except:
        error('Error when scrolling to element!')

    time.sleep(.15)

def get_element(driver:object, value:str, by:str='XPATH')->object:
    """
    Returns a specific web element from a given webdriver by its value and method of searching.

    Arguments:
        driver (object): The webdriver instance to search in.
        value (str): The value to search for.
        by (str, optional): The method of searching. Can be 'XPATH', 'ID', or 'LINK_TEXT'. Defaults to 'XPATH'.

    Returns:
        object: The web element found.
    """

    i = 0
    element = None
    while not element:
        try:
            time.sleep(.5)
            if by == 'XPATH':
                element = driver.find_element(by=By.XPATH, value=value)
            if by == 'ID':
                element = driver.find_element(by=By.ID, value=value)
            if by == 'LINK_TEXT':
                element = driver.find_element(by=By.LINK_TEXT, value=value)

            i += 1
        except:
            i += 1
            continue
        finally:
            if i > 14:
                raise TimeoutError
    
    return element

def check_element(driver:object, xpath:str='', web_element:object=None)->bool:
    """
    Checks the existence and clickability of a specific web element in a given webdriver.

    Arguments:
        driver (object): The webdriver instance to check in.
        xpath (str, optional): The xpath of the element to check. Defaults to ''.
        web_element (object, optional): The web element to check. If not provided, will search for the element using xpath. 

    Returns:
        bool: True if the element exists and is clickable, False otherwise.
    """

    try:
        if xpath != '':
            element = WebDriverWait(driver, timeout=1).until(lambda x: x.find_element(by=By.XPATH, value=xpath))
        else:
            element = web_element

        scroll_to(driver, element)
        if not element.is_displayed():
            raise Exception

        if not element.is_enabled():
            raise Exception

        if xpath != '':
            WebDriverWait(driver, 1).until(EC.element_to_be_clickable((By.XPATH, xpath)))

    except:
        return False
    else:
        return element


#!/usr/bin/env python3

from setuptools import setup, find_packages

setup(
    name='py-easy-scrap',
    version='0.1.3',
    url='https://github.com/ygalvao/py-easy-scrap',
    author='Yuri Henrique Galvao',
    author_email='yuri@galvao.ca',
    description='A useful package for web scraping with Selenium',
    packages=find_packages(),
    install_requires=[
        'selenium',
    ],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
    ],
)
